package com.miaxis.app.controller.dict;

import com.miaxis.common.constant.Constants;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.page.ResponsePageInfo;
import com.miaxis.dict.domain.DictCity;
import com.miaxis.dict.service.IDictCityService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 【全国城市】Controller
 *
 * @author miaxis
 * @date 2023-03-23
 */
@RestController
@RequestMapping(Constants.STUDENT_PREFIX+"/dict/city")
@Api(tags={"【APP-全国城市】"})
public class DictCityController extends BaseController{
    @Autowired
    private IDictCityService dictCityService;

    /**
     * 查询全国城市列表
     */
    @GetMapping("/list")
    @ApiOperation("查询全国城市列表")
        @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum",value = "当前页码" ,dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "pageSize",value = "每页数据量" , dataType = "int", paramType = "query", required = false),
    })
    public ResponsePageInfo<DictCity> list(@ModelAttribute DictCity dictCity){
        startPage();
        List<DictCity> list = dictCityService.selectDictCityList(dictCity);
        return toResponsePageInfo(list);
    }

    /**
     * 查询全国城市列表
     */
    @GetMapping("/getlist")
    @ApiOperation("查询全国城市列表-无分页")
    public Response<List<DictCity>> getlist(@ModelAttribute DictCity dictCity){
        List<DictCity> list = dictCityService.selectDictCityList(dictCity);
        return Response.success(list);
    }


    /**
     * 获取全国城市详细信息
     */
    @GetMapping(value = "/{code}")
    @ApiOperation("获取全国城市详细信息")
    public Response<DictCity> getDictCityByCode(
            @ApiParam(name = "code", value = "全国城市参数", required = true)
            @PathVariable("code") String code
    ){
        return Response.success(dictCityService.getDictCityByCode(code));
    }


}
