package com.miaxis.app.controller.wx;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.miaxis.common.config.WxpayConfig;
import com.miaxis.common.constant.Constants;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.domain.entity.UserInfo;
import com.miaxis.common.enums.OrderStatusEnum;
import com.miaxis.common.exception.CustomException;
import com.miaxis.common.utils.SecurityUtils;
import com.miaxis.common.utils.ServletUtils;
import com.miaxis.common.utils.ip.IpUtils;
import com.miaxis.common.utils.uuid.CommonUtils;
import com.miaxis.system.service.ISysDictDataService;
import com.miaxis.wx.domain.WxOrder;
import com.miaxis.wx.dto.WxOrderDTO;
import com.miaxis.wx.service.IWxOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.security.PrivateKey;
import java.security.Signature;
import java.util.Base64;

import static com.miaxis.common.utils.OrderCodeFactory.getOrderCode;

/**
 * 【小程序-微信支付】Controller
 *
 * @author miaxis
 * @date 2021-03-10
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(Constants.STUDENT_PREFIX+"/wx")
@Api(tags = {"【app-微信支付】"})
@Slf4j
public class WxController extends BaseController {

    @Autowired
    private HttpClient httpClient;

    @Autowired
    private WxpayConfig wxpayConfig;

    @Autowired
    private IWxOrderService wxOrderService;

    @Autowired
    private ISysDictDataService dictDataService;




    @Value("${xcx.appId}")
    private String xcxAppId;

    private String appId;

    private String bundleId;

    private String packageName;

    @Value("${gzh.appId}")
    private String gzhAppId;

    @Value("${wxpay.notifyUrl}")
    private  String notifyUrl ;




    /**
     * APP微信支付获取预订单id
     */
    @PostMapping(value = "/appPrepareOrder")
    @ApiOperation("app端微信支付下单")
    public Response<WxParamEntity> appPrepareOrder(@RequestBody WxOrderDTO wxOrderDTO) throws Exception{

        UserInfo userInfo = SecurityUtils.getLoginUser().getStudent();
        if(wxOrderDTO.getGoodsType()==1) {
            //创建本地微信订单
            WxOrder order = new WxOrder();
            String orderCode = getOrderCode(userInfo.getId());
            order.setGoodsId(wxOrderDTO.getGoodsId());
            order.setGoodsType(1);
            order.setOutTradeNo(orderCode);
            order.setUserId(userInfo.getId());
            order.setXcxOpenid(userInfo.getXcxOpenid());
            order.setAppOpenid(userInfo.getAppOpenid());
            order.setGzhOpenid(userInfo.getGzhOpenid());
            order.setUnionId(userInfo.getUnionId());
            order.setPhoneType(wxOrderDTO.getPhoneType()); //手机类型 1:苹果 2:安卓
            order.setForeType(wxOrderDTO.getForeType()); //前端类型 1:app 2:小程序 3:公众号
            order.setOrderStatus(OrderStatusEnum.PROCESSING.getCode());
            wxOrderService.save(order);
            return Response.success(placeAppWxOrder(order, ""));
        } else if (wxOrderDTO.getGoodsType()==2) {
            throw new CustomException("请正确传入GoodsType，GoodsType=2未开发");
        } else {
            throw new CustomException("请正确传入GoodsType。");
        }

    }


    /**
     * APP下单
     * @param order
     * @param goodsName
     * @return
     * @throws Exception
     */
    private WxParamEntity placeAppWxOrder(WxOrder order, String goodsName) throws Exception {
        HttpPost httpPost = initHttpPost(wxpayConfig.getV3url());
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("mchid",wxpayConfig.getMerchantId())
                .put("appid", appId)
                .put("description", goodsName)
                .put("notify_url", notifyUrl)
                .put("out_trade_no", order.getOutTradeNo());
        rootNode.putObject("amount")
                .put("total", order.getTotal());
        objectMapper.writeValue(bos, rootNode);
        httpPost.setEntity(new StringEntity(rootNode.toString(), "utf-8"));
        HttpResponse response = httpClient.execute(httpPost);
        String bodyAsString = EntityUtils.toString(response.getEntity());
        if (JSONObject.parseObject(bodyAsString).get("prepay_id") == null){
            throw new CustomException(JSONObject.parseObject(bodyAsString).get("message").toString());
        }
        return getWxAppParamJson(bodyAsString);
    }


    /**
     * APP支付生成调用调起微信支付所需参数
     * @param bodyAsString
     * @return
     * @throws Exception
     */
    private WxParamEntity getWxAppParamJson(String bodyAsString) throws Exception {
        WxParamEntity wxParamEntity = new WxParamEntity();
        wxParamEntity.setAppId(appId);
        wxParamEntity.setPartnerId(wxpayConfig.getMerchantId());
        String  prepayId = JSONObject.parseObject(bodyAsString).getString("prepay_id");
        wxParamEntity.setPrepayId(prepayId);
        wxParamEntity.setPackageValue("Sign=WXPay");
        String nonce_str = RandomStringUtils.randomAlphanumeric(32);
        wxParamEntity.setNonceStr(nonce_str);
        String timestamp = String.valueOf(System.currentTimeMillis()/1000);
        wxParamEntity.setTimeStamp(timestamp);

        StringBuffer sb = new StringBuffer();
        sb.append(appId + "\n");
        sb.append(timestamp + "\n");
        sb.append(nonce_str + "\n");
        sb.append(prepayId+ "\n");

        File file = new ClassPathResource("wechatpay/apiclient_key.pem").getFile();
        String realPath =file.getAbsolutePath();
        PrivateKey privateKey = CommonUtils.getPrivateKey(realPath);
        // 进行签名服务
        Signature signature = Signature.getInstance("SHA256withRSA");
        signature.initSign(privateKey);
        signature.update(sb.toString().getBytes("UTF-8"));
        byte[] signedData = signature.sign();
        String base64Str =  Base64.getEncoder().encodeToString(signedData);
        wxParamEntity.setSign(base64Str);

        return wxParamEntity;
    }

    /**
     * APP支付返回参数
     */
    @Data
    class  WxParamEntity{
        String appId;
        String partnerId;
        String prepayId;
        String packageValue;
        String nonceStr;
        String timeStamp;
        String sign;
    }






    /**
     * 微信小程序或公众号内  JSAPI下单
     * @param order
     * @param goodsName
     * @return
     * @throws Exception
     */
    private JSONObject placeXcxWxOrder(WxOrder order, String goodsName) throws Exception {
        HttpPost httpPost = initHttpPost(wxpayConfig.getV3JsUrl());
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("mchid",wxpayConfig.getMerchantId())
                .put("appid", xcxAppId)
                .put("description", goodsName)
                .put("notify_url", notifyUrl)
                .put("out_trade_no", order.getOutTradeNo());
        rootNode.putObject("amount").put("total", order.getTotal());
        rootNode.putObject("payer").put("openid", order.getXcxOpenid());
        objectMapper.writeValue(bos, rootNode);
        httpPost.setEntity(new StringEntity(rootNode.toString(), "utf-8"));
        HttpResponse response = httpClient.execute(httpPost);
        String bodyAsString = EntityUtils.toString(response.getEntity());
        if (JSONObject.parseObject(bodyAsString).get("prepay_id") == null){
            throw new CustomException(JSONObject.parseObject(bodyAsString).get("message").toString());
        }
        return getWxParamJson(bodyAsString,xcxAppId);

    }





    /**
     *
     * @param order
     * @param goodsName
     * @return
     * @throws Exception
     */
    private JSONObject placeGzhJsWxOrder(WxOrder order, String goodsName) throws Exception {
        HttpPost httpPost = initHttpPost(wxpayConfig.getV3JsUrl());
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("mchid",wxpayConfig.getMerchantId())
                .put("appid", gzhAppId)
                .put("description", goodsName)
                .put("notify_url", notifyUrl)
                .put("out_trade_no", order.getOutTradeNo());
        rootNode.putObject("amount").put("total", order.getTotal());
        rootNode.putObject("payer").put("openid", order.getGzhOpenid());
        objectMapper.writeValue(bos, rootNode);
        httpPost.setEntity(new StringEntity(rootNode.toString(), "utf-8"));
        HttpResponse response = httpClient.execute(httpPost);
        String bodyAsString = EntityUtils.toString(response.getEntity());
        if (JSONObject.parseObject(bodyAsString).get("prepay_id") == null){
            throw new CustomException(JSONObject.parseObject(bodyAsString).get("message").toString());
        }
        return getWxParamJson(bodyAsString, gzhAppId);

    }


    /**
     * 微信小程序或公众号内  JSAPI下单
     * 生成调用调起微信支付所需参数
     * @param bodyAsString
     * @return
     * @throws Exception
     */
    private JSONObject getWxParamJson(String bodyAsString,String appId) throws Exception {
        String  packageStr = "prepay_id="+JSONObject.parseObject(bodyAsString).get("prepay_id");
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("package",packageStr);
        String nonce_str = RandomStringUtils.randomAlphanumeric(32);
        jsonObject.put("nonceStr",nonce_str);
        long timestamp =  System.currentTimeMillis()/1000;
        jsonObject.put("timeStamp",String.valueOf(timestamp));
        jsonObject.put("signType","RSA");
        StringBuffer sb = new StringBuffer();
        sb.append(appId + "\n");
        sb.append(timestamp + "\n");
        sb.append(nonce_str + "\n");
        sb.append(packageStr+ "\n");
        System.out.println(sb);

        File file = new ClassPathResource("wechatpay/apiclient_key.pem").getFile();
        String realPath =file.getAbsolutePath();
        PrivateKey privateKey = CommonUtils.getPrivateKey(realPath);
        // 进行签名服务
        Signature signature = Signature.getInstance("SHA256withRSA");
        signature.initSign(privateKey);
        signature.update(sb.toString().getBytes("UTF-8"));
        byte[] signedData = signature.sign();
        String base64Str =  Base64.getEncoder().encodeToString(signedData);
        jsonObject.put("paySign",base64Str);
        jsonObject.put("appId", appId);
        return jsonObject;
    }










    /**
     * 微信H5下单
     * @param order
     * @param goodsName
     * @return
     * @throws Exception
     */
    private JSONObject placeH5WxOrder(WxOrder order, String goodsName) throws Exception {
        HttpPost httpPost = initHttpPost(wxpayConfig.getV3H5Url());
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode rootNode = objectMapper.createObjectNode();
        ObjectNode sceneInfoNode = objectMapper.createObjectNode();
        ObjectNode h5InfoNode = objectMapper.createObjectNode();
        rootNode.put("mchid",wxpayConfig.getMerchantId())
                .put("appid", gzhAppId)
                .put("description", goodsName)
                .put("notify_url", notifyUrl)
                .put("out_trade_no", order.getOutTradeNo());
        rootNode.putObject("amount").put("total", order.getTotal()).put("currency","CNY");

        //前端类型 1:app 2:小程序 3:公众号
        if(order.getPhoneType()==1 && order.getForeType()==1){ //1苹果  又是APP中支付
            h5InfoNode.put("type", "iOS");
            h5InfoNode.put("bundle_id",bundleId);
        } else if (order.getPhoneType()==2 && order.getForeType()==1){ //2安卓 又是APP中支付
            h5InfoNode.put("type", "Android");
            h5InfoNode.put("package_name",packageName);
        } else {
            h5InfoNode.put("type", "Wap");
        }
        String ipAddr = IpUtils.getIpAddr(ServletUtils.getRequest()); //获取请求IP
        sceneInfoNode.put("payer_client_ip", ipAddr)
                .set("h5_info", h5InfoNode);
        rootNode.set("scene_info",sceneInfoNode);

        objectMapper.writeValue(bos, rootNode);
        httpPost.setEntity(new StringEntity(rootNode.toString(), "utf-8"));
        HttpResponse response = httpClient.execute(httpPost);
        String bodyAsString = EntityUtils.toString(response.getEntity());
        //if (JSONObject.parseObject(bodyAsString).get("prepay_id") == null){
        //    throw new CustomException(JSONObject.parseObject(bodyAsString).get("message").toString());
        //}

        return JSONObject.parseObject(bodyAsString);

    }






    private HttpPost initHttpPost(String url) {
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("Accept", "application/json");
        httpPost.addHeader("Content-type","application/json; charset=utf-8");
        return httpPost;
    }



}
