package com.miaxis.pc.controller;

import com.miaxis.common.annotation.Log;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.page.ResponsePageInfo;
import com.miaxis.common.enums.BusinessTypeEnum;
import com.miaxis.common.utils.poi.ExcelUtil;
import com.miaxis.dict.domain.DictCity;
import com.miaxis.dict.service.IDictCityService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * 【全国城市】Controller
 *
 * @author miaxis
 * @date 2023-03-23
 */
@RestController
@RequestMapping("/dict/city")
@Api(tags={"【PC-全国城市】"})
public class PcDictCityController extends BaseController{
    @Autowired
    private IDictCityService dictCityService;

    /**
     * 查询全国城市列表
     */
    @GetMapping("/list")
    @ApiOperation("查询全国城市列表")
        @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum",value = "当前页码" ,dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "pageSize",value = "每页数据量" , dataType = "int", paramType = "query", required = false),
    })
    public ResponsePageInfo<DictCity> list(@ModelAttribute DictCity dictCity){
        startPage();
        List<DictCity> list = dictCityService.selectDictCityList(dictCity);
        return toResponsePageInfo(list);
    }
    


    /**
     * 获取全国城市详细信息
     */
    @GetMapping(value = "/{id}")
    @ApiOperation("获取全国城市详细信息")
    public Response<DictCity> getInfo(
            @ApiParam(name = "id", value = "全国城市参数", required = true)
            @PathVariable("id") Long id
    ){
        return Response.success(dictCityService.getById(id));
    }


}
