package com.miaxis.system.controller.system;

import com.alibaba.fastjson.JSONObject;
import com.miaxis.apple.service.IAppleService;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.domain.WxUserInfo;
import com.miaxis.common.core.domain.entity.SysMenu;
import com.miaxis.common.core.domain.entity.SysUser;
import com.miaxis.common.core.domain.model.LoginBody;
import com.miaxis.common.core.domain.model.LoginBodyNoCode;
import com.miaxis.common.core.domain.model.LoginUser;
import com.miaxis.common.core.domain.model.WxResult;
import com.miaxis.common.enums.StudentLoginTypeEnum;
import com.miaxis.common.exception.CustomException;
import com.miaxis.common.utils.SecurityUtils;
import com.miaxis.common.utils.ServletUtils;
import com.miaxis.framework.web.service.SysLoginService;
import com.miaxis.framework.web.service.SysPermissionService;
import com.miaxis.framework.web.service.TokenService;
import com.miaxis.system.dto.common.RouterDTO;
import com.miaxis.system.dto.system.AppleTokenDTO;
import com.miaxis.system.dto.system.TokenDTO;
import com.miaxis.system.dto.system.UserInfoDTO;
import com.miaxis.system.service.ISysMenuService;
import com.miaxis.user.service.IUserInfoService;
import com.miaxis.wx.service.IWxService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Set;

/**
 * 登录验证
 *
 * @author miaxis
 */
@RestController
@Api(tags={"【系统-登录验证】"})
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private TokenService tokenService;

    @Resource
    private IAppleService appleService;


    @Autowired
    private IWxService wxService;


    @Autowired
    private IUserInfoService userInfoService;


    private static Logger logger = LoggerFactory.getLogger(SysLoginController.class);

    @Value("${gzh.appId}")
    private String gzhAppid;

    @Value("${gzh.appSecret}")
    private String gzhAppSecret;


    @Value("${xcx.appId}")
    private String xcxAppid;

    @Value("${xcx.appSecret}")
    private String xcxAppSecret;







    /**
     * 登录方法 （含验证码）
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    @ApiOperation("系统-登录方法")
    public Response<TokenDTO> login(@RequestBody LoginBody loginBody)
    {   TokenDTO tokenDTO = new TokenDTO();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        tokenDTO.setToken(token);
        return Response.success(tokenDTO);
    }
    /**
     * 登录方法 （不含验证码）
     *
     * @param loginBodyNoCode 登录信息
     * @return 结果
     */
    @PostMapping("/login/noCode")
    @ApiOperation("登录方法（不含验证码）")
    public Response<TokenDTO> loginNoCode(@RequestBody LoginBodyNoCode loginBodyNoCode)
    {   TokenDTO tokenDTO = new TokenDTO();
        // 生成令牌
        String token = loginService.login(loginBodyNoCode.getUsername(), loginBodyNoCode.getPassword());
        tokenDTO.setToken(token);
        return Response.success(tokenDTO);
    }


    /**
     * 用户 openid登录
     *
     */
    @PostMapping("/login/gzhcode")
    @ApiOperation("公众号h5用户授权码模式登录")
    public Response<TokenDTO> loginByAuthorizationCode(String authorizationCode ){
        String wxResultStr = wxService.getWxToken(gzhAppid, gzhAppSecret, authorizationCode, "authorization_code");
        logger.info("微信授权码登录返回值:"+wxResultStr);
        WxResult wxResult = JSONObject.parseObject(wxResultStr,WxResult.class);

        // 生成令牌
        if (wxResult.getErrcode() != null){
            throw new CustomException("微信授权无效，请重新授权");
        }
        String userInfoStr = wxService.getUserInfo("zh_CN", wxResult.getAccess_token(), wxResult.getOpenid());
        WxUserInfo wxUserInfo = JSONObject.parseObject(userInfoStr,WxUserInfo.class);
        ServletUtils.getRequest().setAttribute("headImage",wxUserInfo.getHeadimgurl());
        ServletUtils.getRequest().setAttribute("nickName",wxUserInfo.getNickname());
        ServletUtils.getRequest().setAttribute("openid",wxUserInfo.getOpenid());
        String token = loginService.login(wxUserInfo.getUnionid(),null, StudentLoginTypeEnum.AUTHORIZATION_H5_LOGIN.getCode());
        TokenDTO tokenDTO = new TokenDTO();
        tokenDTO.setToken(token);
        tokenDTO.setThirdResult(wxUserInfo);
        return Response.success(tokenDTO) ;



    }


    /**
     * 用户 jscode登录
     *
     */
    @PostMapping("/login/jscode")
    @ApiOperation("小程序jscode登录")
    public Response<TokenDTO> getInfo(String jscode,String pUnionId){
        String result = wxService.getWxInfo(xcxAppid, xcxAppSecret, jscode,"authorization_code");
        JSONObject jsonString = JSONObject.parseObject(result);
        WxResult wxResult = JSONObject.toJavaObject(jsonString, WxResult.class);
        TokenDTO tokenDTO = new TokenDTO();
        // 生成令牌
        if (wxResult.getOpenid() != null){
            // 生成令牌
            WxUserInfo wxUserInfo = new WxUserInfo();
            BeanUtils.copyProperties(wxResult,wxUserInfo);
            ServletUtils.getRequest().setAttribute("headImage",wxUserInfo.getHeadimgurl());
            ServletUtils.getRequest().setAttribute("nickName",wxUserInfo.getNickname());
            ServletUtils.getRequest().setAttribute("openid",wxUserInfo.getOpenid());
            ServletUtils.getRequest().setAttribute("xcxOpenid",wxUserInfo.getOpenid());
            ServletUtils.getRequest().setAttribute("pUnionId",pUnionId);
            String token = loginService.login(wxUserInfo.getUnionid(),null, StudentLoginTypeEnum.AUTHORIZATION_XCX_LOGIN.getCode());
            tokenDTO.setToken(token);
            tokenDTO.setThirdResult(wxUserInfo);
            return Response.success(tokenDTO) ;
        }
        else{
            return Response.error(500,wxResult.getErrmsg());
        }

    }


    @PostMapping("/login/code/test")
    @ApiOperation("通用微信用户授权码模式登录--测试")
    public Response<TokenDTO> testloginByAuthorizationCode(String authorizationCode ){
        WxUserInfo wxUserInfo = new WxUserInfo();
       // wxUserInfo.setOpenid("ovKTX5-FKLF6_sgTtCIXpG_lz3PY");

        ServletUtils.getRequest().setAttribute("headImage","https://thirdwx.qlogo.cn/mmopen/vi_32/Q0j4TwGTfTLwSoTjHPX5tia8srBKHGGjtaAmvj90UmvQex5kJpdP5TmxF5Diar8uxRHFkSTR2SDqvj3ej6YXNcDQ/132");
        ServletUtils.getRequest().setAttribute("nickName","\uD83D\uDE0DQing");
        //ServletUtils.getRequest().setAttribute("third","weixin");
        //ServletUtils.getRequest().setAttribute("openid",wxUserInfo.getOpenid());
        String token = loginService.login("o_cXm6Mf4HMVN0J64GfHPK4POZng",null, StudentLoginTypeEnum.AUTHORIZATION_CODE_LOGIN.getCode());
        TokenDTO tokenDTO = new TokenDTO();
        tokenDTO.setToken(token);
        return Response.success(tokenDTO) ;
    }



    @PostMapping("/login/code/ios")
    @ApiOperation("IOS登录")
    public Response<AppleTokenDTO> ioslogin(String identityToken){
        try {
            JSONObject appleUser = appleService.getAppleUserInfo(identityToken);
            String token = loginService.login(appleUser.getString("sub"),null, StudentLoginTypeEnum.IOS_CODE_LOGIN.getCode());
            AppleTokenDTO tokenDTO = new AppleTokenDTO();
            tokenDTO.setAppleUserInfo(appleUser);
            tokenDTO.setToken(token);
            return Response.success(tokenDTO);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException("获取苹果用户信息失败");
        }


    }

    @PostMapping("/login/app/deletion")
    @ApiOperation("app用户注销(删号)")
    public Response deletion(){
        LoginUser loginUser = SecurityUtils.getLoginUser();
        // 删除用户缓存记录
        tokenService.delLoginUser(loginUser.getToken());
        Long id  = loginUser.getStudent().getId();
        //删除用户
        userInfoService.removeById(id);
        return Response.success();

    }



    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getInfo")
    @ApiOperation("获取用户信息")
    public Response<UserInfoDTO> getInfo()
    {
        SecurityUtils.getLoginUser();
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        UserInfoDTO userInfoDTO = new UserInfoDTO();
        SysUser user = loginUser.getUser();
        if (user != null){
            userInfoDTO.setUser(user);
            // 角色集合
            Set<String> roles = permissionService.getRolePermission(user);
            // 权限集合
            Set<String> permissions = permissionService.getMenuPermission(user);
            userInfoDTO.setRoles(roles);
            userInfoDTO.setPermissions(permissions);
        }
        userInfoDTO.setStudent(loginUser.getStudent());
        return Response.success(userInfoDTO);
    }

    /**
     * 获取路由信息
     *
     * @return 路由信息
     */
    @GetMapping("getRouters")
    @ApiOperation("获取路由信息")
    public Response<List<RouterDTO>> getRouters()
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        // 用户信息
        SysUser user = loginUser.getUser();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(user.getUserId());
        return Response.success(menuService.buildMenus(menus));
    }
}
