package com.miaxis.test;

import com.miaxis.XpgxApplication;
import com.miaxis.dict.domain.DictCity;
import com.miaxis.dict.service.IDictCityService;
import net.sourceforge.pinyin4j.PinyinHelper;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.Collator;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;

@ActiveProfiles("dev")
@SpringBootTest(classes = XpgxApplication.class)
@RunWith(SpringRunner.class)
public class ReadFileNameTest2 {


    @Autowired
    private IDictCityService dictCityService;


    /**
     * 用于通过文件夹导出考场信息
     */
    @Test
    public void test() {
        String folderPath = "H:/广东省"; // 替换为你的文件夹路径
        String excelFilePath = "H:/exam_info.xlsx"; // 替换为你的Excel文件路径

        try {
            Workbook workbook = new XSSFWorkbook();
            Sheet sheet = workbook.createSheet("Folders");

            // 创建表头
            Row headerRow = sheet.createRow(0);
            headerRow.createCell(0).setCellValue("id");
            headerRow.createCell(1).setCellValue("name");
            headerRow.createCell(2).setCellValue("image");
            headerRow.createCell(3).setCellValue("province_id");
            headerRow.createCell(4).setCellValue("province");
            headerRow.createCell(5).setCellValue("city_id");
            headerRow.createCell(6).setCellValue("city");
            headerRow.createCell(7).setCellValue("county_id");
            headerRow.createCell(8).setCellValue("county");
            headerRow.createCell(9).setCellValue("price");


            // 获取根文件夹
            File rootFolder = new File(folderPath);

            // 递归遍历文件夹
            traverseFolders(rootFolder, sheet, 1);

            // 将工作簿写入Excel文件
            FileOutputStream outputStream = new FileOutputStream(excelFilePath);
            workbook.write(outputStream);
            workbook.close();
            outputStream.close();

            System.out.println("Excel文件生成成功！");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void traverseFolders(File folder, Sheet sheet, int rowIndex) {
        String province = folder.getName();
        String provinceId = "44";
        String city = null;
        String cityId = null;
        String county = null;
        String countyId = null;
        String exam = null;

        if (folder.isDirectory()) {
            File[] files = this.sortFiles(folder.listFiles());
            //File[] files = folder.listFiles();
            for (int i = 0; i < files.length; i++) {    //市级
                // System.out.println(files[i].getName());
                city = files[i].getName();
                List<DictCity> dictCityList = dictCityService.getDictCityByName(city);

                for (int j = 0; j < dictCityList.size(); j++) {
                    cityId = dictCityList.get(j).getCode();
                    if (cityId.startsWith("44")) {
                        break;
                    }
                }


                if (files[i].isDirectory()) {
                    File[] files2 = this.sortFiles(files[i].listFiles());
                    //File[] files2 = files[i].listFiles();
                    for (int j = 0; j < files2.length; j++) {   //区级
                        //   System.out.println("    "+files2[j].getName());
                        county = files2[j].getName();
                        List<DictCity> dictCity2List = dictCityService.getDictCityByName(county);

                        for (int x = 0; x < dictCity2List.size(); x++) {
                            countyId = dictCity2List.get(x).getCode();
                            if (countyId.startsWith(cityId)) {
                                break;
                            }
                        }

                        if (files2[j].isDirectory()) {
                            File[] files3 = this.sortFiles(files2[j].listFiles());
                            //File[] files3 = files2[j].listFiles();
                            for (int k = 0; k < files3.length; k++) { //考场
                                //   System.out.println("        "+files3[k].getName());
                                exam = files3[k].getName();

                                //写入excel
                                Row row = sheet.createRow(rowIndex);
                                row.createCell(0).setCellValue("");
                                row.createCell(1).setCellValue(exam);
                                row.createCell(2).setCellValue("");
                                row.createCell(3).setCellValue(provinceId);
                                row.createCell(4).setCellValue(province);
                                row.createCell(5).setCellValue(cityId);
                                row.createCell(6).setCellValue(city);
                                row.createCell(7).setCellValue(countyId);
                                row.createCell(8).setCellValue(county);
                                row.createCell(9).setCellValue(1);

                               // System.out.println(province + "   " + city + "    " + county + "  " + exam);
                                rowIndex++;

                            }
                        }
                    }
                }
            }

        }
    }


    public static File[] sortFiles(File[] files) {
        if (files != null) {
            Collator collator = Collator.getInstance(Locale.CHINA);
            // 使用Comparator.comparing方法结合Lambda表达式按文件名的拼音首字母排序
            //Arrays.sort(files, Comparator.comparing(file -> file.getName(), collator));
            // 使用Pinyin4j将汉字转换为拼音，并按拼音进行排序
            Arrays.sort(files, Comparator.comparing(file -> getPinyin(file.getName()), collator));
        }
        return files;
    }

    // 使用Pinyin4j将汉字转换为拼音
    private static String getPinyin(String hanzi) {
        StringBuilder pinyin = new StringBuilder();
        for (char ch : hanzi.toCharArray()) {
            String[] pinyinArray = PinyinHelper.toHanyuPinyinStringArray(ch);
            if (pinyinArray != null && pinyinArray.length > 0) {
                pinyin.append(pinyinArray[0]);
            } else {
                // 如果无法获取拼音，则原样输出汉字
                pinyin.append(ch);
            }
        }
        return pinyin.toString();
    }


}


