package com.miaxis.common.core.domain.model;


import com.fasterxml.jackson.annotation.JsonIgnore;
import com.miaxis.common.core.domain.entity.SysUser;
import com.miaxis.common.core.domain.entity.UserInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.*;

/**
 * 登录用户身份权限
 *
 * @author miaxis
 */
@ApiModel(value = "LoginUser", description = "登录用户身份权限")
public class LoginUser implements UserDetails
{
    private static final long serialVersionUID = 1L;



    /**
     * 用户唯一标识
     */
    @ApiModelProperty(value = "用户唯一标识")
    private String id;


    /**
     * 用户token
     */
    @ApiModelProperty(value = "token")
    private String token;

    /**
     * 登录时间
     */
    @ApiModelProperty(value = "登录时间")
    private Long loginTime;

    /**
     * 过期时间
     */
    @ApiModelProperty(value = "过期时间")
    private Long expireTime;

    /**
     * 登录IP地址
     */
    @ApiModelProperty(value = "登录IP地址")
    private String ipaddr;

    /**
     * 登录地点
     */
    @ApiModelProperty(value = "登录地点")
    private String loginLocation;

    /**
     * 浏览器类型
     */
    @ApiModelProperty(value = "浏览器类型")
    private String browser;

    /**
     * 操作系统
     */
    @ApiModelProperty(value = "操作系统")
    private String os;

    /**
     * 权限列表
     */
    @ApiModelProperty(value = "权限列表")
    private Set<String> permissions;

    /**
     * 用户信息
     */
    @ApiModelProperty(value = "用户信息")
    private SysUser user;

    /**
     * 学员信息
     */
    @ApiModelProperty(value = "学员信息")
    private UserInfo student;

    public LoginUser(UserInfo student,String loginType) {
        this.student = student;
        this.id= "student:"+student.getId()+"_"+loginType;
    }

    public LoginUser(UserInfo student) {
        this.student = student;
        this.id= "student:"+student.getId();
    }


    public String getToken()
    {
        return token;
    }
    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }
    public void setToken(String token)
    {
        this.token = token;
    }

    public LoginUser()
    {
    }

    public LoginUser(SysUser user, Set<String> permissions)
    {
        this.user = user;
        this.permissions = permissions;
        this.id = "sysUser:"+user.getUserId();
    }

    @JsonIgnore
    @Override
    public String getPassword()
    {
        if (user != null){
            return user.getPassword();
        }else{
            return null;
        }

    }

    @Override
    public String getUsername()
    {
        if (user != null){
            return user.getUserName();
        }else{
            return student.getPhone();
        }
    }

    /**
     * 账户是否未过期,过期无法验证
     */
    @JsonIgnore
    @Override
    public boolean isAccountNonExpired()
    {
        return true;
    }

    /**
     * 指定用户是否解锁,锁定的用户无法进行身份验证
     *
     * @return
     */
    @JsonIgnore
    @Override
    public boolean isAccountNonLocked()
    {
        return true;
    }

    /**
     * 指示是否已过期的用户的凭据(密码),过期的凭据防止认证
     *
     * @return
     */
    @JsonIgnore
    @Override
    public boolean isCredentialsNonExpired()
    {
        return true;
    }

    /**
     * 是否可用 ,禁用的用户不能身份验证
     *
     * @return
     */
    @JsonIgnore
    @Override
    public boolean isEnabled()
    {
        return true;
    }

    public Long getLoginTime()
    {
        return loginTime;
    }

    public void setLoginTime(Long loginTime)
    {
        this.loginTime = loginTime;
    }

    public String getIpaddr()
    {
        return ipaddr;
    }

    public void setIpaddr(String ipaddr)
    {
        this.ipaddr = ipaddr;
    }

    public String getLoginLocation()
    {
        return loginLocation;
    }

    public void setLoginLocation(String loginLocation)
    {
        this.loginLocation = loginLocation;
    }

    public String getBrowser()
    {
        return browser;
    }

    public void setBrowser(String browser)
    {
        this.browser = browser;
    }

    public String getOs()
    {
        return os;
    }

    public void setOs(String os)
    {
        this.os = os;
    }

    public Long getExpireTime()
    {
        return expireTime;
    }

    public void setExpireTime(Long expireTime)
    {
        this.expireTime = expireTime;
    }

    public Set<String> getPermissions()
    {
        return permissions;
    }

    public void setPermissions(Set<String> permissions)
    {
        this.permissions = permissions;
    }

    public SysUser getUser()
    {
        return user;
    }

    public void setUser(SysUser user)
    {
        this.user = user;
    }
    public UserInfo getStudent()
    {
        return this.student;
    }

    public void setStudent(UserInfo student)
    {
        this.student = student;
    }
    @Override
    public Collection<? extends GrantedAuthority> getAuthorities()
    {
        if (this.getStudent() != null){
            List<SimpleGrantedAuthority> roleList = new ArrayList<SimpleGrantedAuthority>();
            roleList.add(new SimpleGrantedAuthority("ROLE_STUDENT"));
            //如果已激活，是vip的话
            Date now = new Date();
            if (this.getStudent().getExpireTime() != null && now.compareTo(this.getStudent().getExpireTime())<0){
                roleList.add(new SimpleGrantedAuthority("ROLE_VIP"));
            }
            return roleList;
        }else {
            return null;
        }


    }
    public void setAuthorities(Collection<? extends GrantedAuthority> collection){

    }

}
