package com.miaxis.framework.web.service;

import com.miaxis.common.core.domain.entity.SysUser;
import com.miaxis.common.core.domain.entity.UserInfo;
import com.miaxis.common.core.domain.model.LoginUser;
import com.miaxis.common.enums.StudentLoginTypeEnum;
import com.miaxis.common.enums.UserStatusEnum;
import com.miaxis.common.exception.BaseException;
import com.miaxis.common.exception.CustomException;
import com.miaxis.common.utils.RandomNameUtils;
import com.miaxis.common.utils.ServletUtils;
import com.miaxis.common.utils.StringUtils;
import com.miaxis.system.service.ISysConfigService;
import com.miaxis.system.service.ISysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

/**
 * 用户验证处理
 *
 * @author miaxis
 */
@Service
public class UserDetailsServiceImpl implements UserDetailsService
{
    private static final Logger log = LoggerFactory.getLogger(UserDetailsServiceImpl.class);

    @Autowired
    private ISysUserService userService;


    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private ISysConfigService configService;

    @Override
    @Transactional(isolation = Isolation.READ_UNCOMMITTED)
    public UserDetails loadUserByUsername(String identification) throws UsernameNotFoundException {
        //获取登录类型
        String loginType = (String) ServletUtils.getRequest().getAttribute("loginType");
        if (StringUtils.isEmpty(loginType)){
            SysUser user = userService.selectUserByUserName(identification);
            if (StringUtils.isNull(user))
            {
                log.info("登录用户：{} 不存在.", identification);
                throw new UsernameNotFoundException("登录用户：" + identification + " 不存在");
            }
            else if (UserStatusEnum.DELETED.getCode().equals(user.getDelFlag()))
            {
                log.info("登录用户：{} 已被删除.", identification);
                throw new BaseException("对不起，您的账号：" + identification + " 已被删除");
            }
            else if (UserStatusEnum.DISABLE.getCode().equals(user.getStatus()))
            {
                log.info("登录用户：{} 已被停用.", identification);
                throw new BaseException("对不起，您的账号：" + identification + " 已停用");
            }
            return createLoginUser(user);
        } else if (StudentLoginTypeEnum.AUTHORIZATION_CODE_LOGIN.getCode().equals(loginType)){
            {
                UserInfo userInfoGetByUnionId = userService.getStudentByUnionId(identification);
                String openid = (String)ServletUtils.getRequest().getAttribute("openid");
                if (userInfoGetByUnionId == null ) {
                    UserInfo userInfo = new UserInfo();
                    userInfo.setHeadImage((String) ServletUtils.getRequest().getAttribute("headImage"));
                    userInfo.setNickName((String) ServletUtils.getRequest().getAttribute("nickName"));
                    userInfo.setAppOpenid(openid);
                    userInfo.setUnionId(identification);
                    userService.saveUserInfo(userInfo);
                    return createLoginUser(userService.getStudentByUnionId(identification),"app");
                }else {
                    UserInfo userInfo = userInfoGetByUnionId ;
                    userInfo.setHeadImage((String) ServletUtils.getRequest().getAttribute("headImage"));
                    userInfo.setNickName((String) ServletUtils.getRequest().getAttribute("nickName"));
                    userInfo.setAppOpenid(openid);
                    userService.updateStudent(userInfo);
                    return createLoginUser(userService.getStudentByUnionId(identification),"app");
                }

            }
        }
        else if (StudentLoginTypeEnum.AUTHORIZATION_H5_LOGIN.getCode().equals(loginType)){
            {
                UserInfo userInfoGetByUnionId = userService.getStudentByUnionId(identification);
                String openid = (String)ServletUtils.getRequest().getAttribute("openid");
                if (userInfoGetByUnionId == null ) {
                    UserInfo userInfo = new UserInfo();
                    userInfo.setHeadImage((String) ServletUtils.getRequest().getAttribute("headImage"));
                    userInfo.setNickName((String) ServletUtils.getRequest().getAttribute("nickName"));
                    userInfo.setGzhOpenid(openid);
                    userInfo.setUnionId(identification);
                    userService.saveUserInfo(userInfo);
                    return createLoginUser(userService.getStudentByUnionId(identification),"GH5");
                }else {
                    UserInfo userInfo = userInfoGetByUnionId ;
                    userInfo.setHeadImage((String) ServletUtils.getRequest().getAttribute("headImage"));
                    userInfo.setNickName((String) ServletUtils.getRequest().getAttribute("nickName"));
                    userInfo.setGzhOpenid(openid);
                    userService.updateStudent(userInfo);
                    return createLoginUser(userService.getStudentByUnionId(identification),"GH5");
                }

            }
        }
        else if (StudentLoginTypeEnum.AUTHORIZATION_XCX_LOGIN.getCode().equals(loginType)){
            {
                UserInfo userInfoGetByUnionId = userService.getStudentByUnionId(identification);
                String openid = (String)ServletUtils.getRequest().getAttribute("openid");
                String pUnionId = (String)ServletUtils.getRequest().getAttribute("pUnionId");
                UserInfo userInfoGetByOpenId = userService.getStudentByOpenId(openid);
                if (userInfoGetByUnionId == null ) {
                    UserInfo userInfo = new UserInfo();
                    userInfo.setHeadImage("https://mmbiz.qpic.cn/sz_mmbiz_jpg/U1q0AYGzob5AbSPdqDs1tbOATPL8gJfbiaaJbCf0a7ibzBAyca1eoDvFgzGQc6GeuG1icBiaqevcFW1k5no8accn0A/640?wx_fmt=jpeg");
                    userInfo.setNickName(RandomNameUtils.generateName());
                    userInfo.setXcxOpenid(openid);
                    userInfo.setUnionId(identification);
                    userInfo.setPUnionId(pUnionId);
                    userService.saveUserInfo(userInfo);
                    return createLoginUser(userService.getStudentByUnionId(identification),"XCX");
                }else {
                    UserInfo userInfo = userInfoGetByUnionId ;
                    userInfo.setXcxOpenid(openid);
                    userService.updateStudent(userInfo);
                    return createLoginUser(userService.getStudentByUnionId(identification),"XCX");
                }

            }
        }

        else if (StudentLoginTypeEnum.IOS_CODE_LOGIN.getCode().equals(loginType)){
            {
                UserInfo userInfo = userService.getStudentByOpenId(identification);
                if (userInfo == null) {
                    userInfo = new UserInfo();
                    userInfo.setAppOpenid(identification);
                    userService.saveUserInfo(userInfo);
                }
                return createLoginUser(userService.getStudentByOpenId(identification),"XCX");
            }
        }
        else{
            throw new CustomException("登录类型不存在");
        }



    }

    private UserDetails createLoginUser(UserInfo student,String loginType) {
        return new LoginUser(student,loginType);
    }

    public UserDetails createLoginUser(SysUser user)
    {
        return new LoginUser(user, permissionService.getMenuPermission(user));
    }
}
