package com.miaxis.wx.service;


import com.miaxis.common.config.FeignConfig;
import com.miaxis.wx.dto.*;
import feign.Response;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 *
 * 微信接口
 */
@FeignClient(name="wxService",
        url = "https://api.weixin.qq.com",configuration = FeignConfig.class)
@Component
public interface IWxService {
    /**
     *  获取openid
     * @param appid
     * @param secret
     * @param code
     * @param grantType
     * @return
     */
    @GetMapping(value = "/sns/oauth2/access_token")
    String getWxToken(
            @RequestParam("appid") String appid,
            @RequestParam("secret") String secret,
            @RequestParam("code") String code,
            @RequestParam("grant_type") String grantType);


    /**
     * 公众号获取用户信息
     */
    @GetMapping(value = "/sns/userinfo")
    String getUserInfo(
            @RequestParam("lang") String lang,
            @RequestParam("access_token") String accessToken,
            @RequestParam("openid") String openid
    );


    /**
     * 获取接口调用凭据（小程序）
     */
    @GetMapping(value = "/cgi-bin/token")
    String getXcxToken(
            @RequestParam("grant_type") String grantType,
            @RequestParam("appid") String appid,
            @RequestParam("secret") String secret
    );

    /**
     * 获取稳定版接口调用凭据（小程序）
     */
    @PostMapping(value = "/cgi-bin/stable_token")
    String getXcxStableToken(WxStableTokenDTO wxStableTokenDTO);


    /**
     * 获取手机号 (小程序)
     * @param access_token
     * @param wxGetPhoneDTO
     * @return
     */
    @PostMapping(value = "/wxa/business/getuserphonenumber")
    String getUserPhoneNumber(
            @RequestParam("access_token") String access_token,
            WxGetPhoneDTO wxGetPhoneDTO
    );

    /**
     * 获取不限制的小程序码
     * @param access_token
     * @return
     */
    @PostMapping(value = "/wxa/getwxacodeunlimit")
    Response getWxacodeunLimit(@RequestParam("access_token") String access_token, WxacodeUnlimitDTO wxacodeUnlimitDTO);


    /**
     *  小程序获取openid
     * @param appid
     * @param secret
     * @param jsCode
     * @param grantType
     * @return
     */
    @GetMapping(value = "/sns/jscode2session")
    String getWxInfo(
            @RequestParam("appid") String appid,
            @RequestParam("secret") String secret,
            @RequestParam("js_code") String jsCode,
            @RequestParam("grant_type") String grantType);

    /**
     * 获取jsapi_ticket
     */
    @GetMapping(value = "/cgi-bin/ticket/getticket")
    String getticket(
            @RequestParam("access_token") String accessToken,
            @RequestParam("type") String type
    );

    /**
     * 调用公众号素材列表
     * @param accessToken
     * @param wxMaterial
     * @return
     */
    @PostMapping(value = "/cgi-bin/material/batchget_material")
    String getBatchMateria(
            @RequestParam("access_token") String accessToken, WxMaterialList wxMaterial);

    /**
     * 调用已发布文章列表
     */
    @PostMapping(value = "/cgi-bin/freepublish/batchget")
    String getFreepublish(@RequestParam("access_token") String accessToken, WxFreepublish wxFreepublish);



    /**
     * 调用公众号单个素材
     * @param accessToken
     * @param wxMaterial
     * @return
     */
    @PostMapping(value = "/cgi-bin/material/get_material")
    String getMateria(@RequestParam("access_token") String accessToken, WxMaterial wxMaterial);


    /**
     * 清空api的调用quota
     */
    @PostMapping(value = "cgi-bin/clear_quota")
    String clearQuota(@RequestParam("access_token") String accessToken, Map<String,String> params);

}
