package com.miaxis.wx.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.miaxis.common.constant.Constants;
import com.miaxis.common.core.redis.RedisCache;
import com.miaxis.wx.dto.WxStableTokenDTO;
import com.miaxis.wx.service.IWxXcxService;
import com.miaxis.wx.service.IWxService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

@Service
@Slf4j
@RequiredArgsConstructor
public class WxXcxServiceImpl implements IWxXcxService {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IWxService wxService;

    @Value("${xcx.appId}")
    private String xcxAppId;

    @Value("${xcx.appSecret}")
    private String appSecret;

    @Override
    public String getLocalXcxToken() {
        Object cacheXcxToken= redisCache.getCacheObject(Constants.XCX_MESSAGE_TOKEN);
        //如果过期，重新获取并保存到redis
        if (cacheXcxToken == null){
            WxStableTokenDTO stableTokenDTO = new WxStableTokenDTO();
            stableTokenDTO.setGrant_type("client_credential");
            stableTokenDTO.setAppid(xcxAppId);
            stableTokenDTO.setSecret(appSecret);

            String result = wxService.getXcxStableToken(stableTokenDTO);
            JSONObject json = JSONObject.parseObject(result);
            String token = json.getString("access_token");
            int expiresIn = json.getIntValue("expires_in");
            redisCache.setCacheObject(Constants.XCX_MESSAGE_TOKEN,token,expiresIn, TimeUnit.SECONDS);
            return token;
        }else {
            String token = String.valueOf(cacheXcxToken);
            return token;
        }
    }


}
